@@ -31,7 +31,10 @@ class TestPasswordsUtil(TestCase):
     @mock.patch("os.path.isfile", return_value=False)
     @mock.patch("passlib.utils.generate_password",
                 return_value="PASSWORD")
-    def test_generate_passwords(self, generate_password_mock, isfile_mock):
+    @mock.patch("tripleoclient.utils.create_cephx_key",
+                return_value="CEPHX_KEY")
+    def test_generate_passwords(self, create_cephx_key_mock,
+                                generate_password_mock, isfile_mock):
 
         mock_open = mock.mock_open()
 
@@ -43,8 +46,13 @@ class TestPasswordsUtil(TestCase):
             mock.call('OVERCLOUD_ADMIN_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_ADMIN_TOKEN=PASSWORD\n'),
             mock.call('OVERCLOUD_AODH_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_BARBICAN_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_CEILOMETER_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_CEILOMETER_SECRET=PASSWORD\n'),
+            mock.call('OVERCLOUD_CEPH_ADMIN_KEY=CEPHX_KEY\n'),
+            mock.call('OVERCLOUD_CEPH_CLIENT_KEY=CEPHX_KEY\n'),
+            mock.call('OVERCLOUD_CEPH_MON_KEY=CEPHX_KEY\n'),
+            mock.call('OVERCLOUD_CEPH_RGW_KEY=CEPHX_KEY\n'),
             mock.call('OVERCLOUD_CINDER_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_DEMO_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_GLANCE_PASSWORD=PASSWORD\n'),
@@ -52,6 +60,9 @@ class TestPasswordsUtil(TestCase):
             mock.call('OVERCLOUD_HAPROXY_STATS_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_HEAT_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_HEAT_STACK_DOMAIN_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_IRONIC_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_MANILA_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_MISTRAL_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_MYSQL_CLUSTERCHECK_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_NEUTRON_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_NOVA_PASSWORD=PASSWORD\n'),
@@ -60,9 +71,12 @@ class TestPasswordsUtil(TestCase):
             mock.call('OVERCLOUD_SAHARA_PASSWORD=PASSWORD\n'),
             mock.call('OVERCLOUD_SWIFT_HASH=PASSWORD\n'),
             mock.call('OVERCLOUD_SWIFT_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_TROVE_PASSWORD=PASSWORD\n'),
+            mock.call('OVERCLOUD_ZAQAR_PASSWORD=PASSWORD\n'),
         ]
         self.assertEqual(sorted(mock_open().write.mock_calls), mock_calls)
-        self.assertEqual(generate_password_mock.call_count, len(mock_calls))
+        self.assertEqual(generate_password_mock.call_count +
+                         create_cephx_key_mock.call_count, len(mock_calls))
 
         self.assertEqual(len(passwords), len(mock_calls))
 
@@ -77,13 +91,21 @@ class TestPasswordsUtil(TestCase):
     @mock.patch("os.path.isfile", return_value=True)
     @mock.patch("passlib.utils.generate_password",
                 return_value="PASSWORD")
-    def test_load_passwords(self, generate_password_mock, isfile_mock):
+    @mock.patch("tripleoclient.utils.create_cephx_key",
+                return_value="CEPHX_KEY")
+    def test_load_passwords(self, create_cephx_key_mock,
+                            generate_password_mock, isfile_mock):
         PASSWORDS = [
             'OVERCLOUD_ADMIN_PASSWORD=PASSWORD\n',
             'OVERCLOUD_ADMIN_TOKEN=PASSWORD\n',
             'OVERCLOUD_AODH_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_BARBICAN_PASSWORD=PASSWORD\n',
             'OVERCLOUD_CEILOMETER_PASSWORD=PASSWORD\n',
             'OVERCLOUD_CEILOMETER_SECRET=PASSWORD\n',
+            'OVERCLOUD_CEPH_ADMIN_KEY=CEPHX_KEY\n',
+            'OVERCLOUD_CEPH_CLIENT_KEY=CEPHX_KEY\n',
+            'OVERCLOUD_CEPH_MON_KEY=CEPHX_KEY\n',
+            'OVERCLOUD_CEPH_RGW_KEY=CEPHX_KEY\n',
             'OVERCLOUD_CINDER_PASSWORD=PASSWORD\n',
             'OVERCLOUD_DEMO_PASSWORD=PASSWORD\n',
             'OVERCLOUD_GLANCE_PASSWORD=PASSWORD\n',
@@ -91,6 +113,9 @@ class TestPasswordsUtil(TestCase):
             'OVERCLOUD_HAPROXY_STATS_PASSWORD=PASSWORD\n',
             'OVERCLOUD_HEAT_PASSWORD=PASSWORD\n',
             'OVERCLOUD_HEAT_STACK_DOMAIN_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_IRONIC_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_MANILA_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_MISTRAL_PASSWORD=PASSWORD\n',
             'OVERCLOUD_MYSQL_CLUSTERCHECK_PASSWORD=PASSWORD\n',
             'OVERCLOUD_NEUTRON_PASSWORD=PASSWORD\n',
             'OVERCLOUD_NOVA_PASSWORD=PASSWORD\n',
@@ -99,9 +124,10 @@ class TestPasswordsUtil(TestCase):
             'OVERCLOUD_SAHARA_PASSWORD=PASSWORD\n',
             'OVERCLOUD_SWIFT_HASH=PASSWORD\n',
             'OVERCLOUD_SWIFT_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_TROVE_PASSWORD=PASSWORD\n',
+            'OVERCLOUD_ZAQAR_PASSWORD=PASSWORD\n',
             'NEUTRON_METADATA_PROXY_SHARED_SECRET=PASSWORD\n',
         ]
-
         mock_open = mock.mock_open(read_data=''.join(PASSWORDS))
         mock_open.return_value.__iter__ = lambda self: self
         mock_open.return_value.__next__ = lambda self: self.readline()
@@ -218,75 +244,6 @@ class TestWaitForStackUtil(TestCase):
 
         self.assertEqual(2, sleep_mock.call_count)
 
-
-class TestWaitForIntrospection(TestCase):
-
-    def test_wait_for_introspection_success(self):
-
-        mock_inspector = mock.Mock()
-        self.node_uuids = [
-            'NODE1',
-            'NODE2',
-        ]
-
-        mock_inspector.get_status.return_value = {
-            'finished': True,
-            'error': None
-        }
-
-        result = utils.wait_for_node_introspection(mock_inspector,
-                                                   self.node_uuids,
-                                                   loops=4, sleep=0.01)
-
-        self.assertEqual(list(result), [
-            ('NODE1', {'error': None, 'finished': True}),
-            ('NODE2', {'error': None, 'finished': True})
-        ])
-
-    def test_wait_for_introspection_partial_success(self):
-
-        mock_inspector = mock.Mock()
-        self.node_uuids = [
-            'NODE1',
-            'NODE2',
-        ]
-
-        mock_inspector.get_status.side_effect = [{
-            'finished': True,
-            'error': None
-        }, {
-            'finished': True,
-            'error': "Failed"
-        }]
-
-        result = utils.wait_for_node_introspection(mock_inspector,
-                                                   self.node_uuids,
-                                                   loops=4, sleep=0.01)
-
-        self.assertEqual(list(result), [
-            ('NODE1', {'error': None, 'finished': True}),
-            ('NODE2', {'error': "Failed", 'finished': True})
-        ])
-
-    def test_wait_for_introspection_timeout(self):
-
-        mock_inspector = mock.Mock()
-        self.node_uuids = [
-            'NODE1',
-            'NODE2',
-        ]
-
-        mock_inspector.get_status.return_value = {
-            'finished': False,
-            'error': None
-        }
-
-        result = utils.wait_for_node_introspection(mock_inspector,
-                                                   self.node_uuids,
-                                                   loops=4, sleep=0.01)
-
-        self.assertEqual(list(result), [])
-
     def test_create_environment_file(self):
 
         json_file_path = "env.json"
@@ -336,14 +293,26 @@ class TestWaitForIntrospection(TestCase):
         self.assertEqual(value, "pa$$word")
 
     @mock.patch("six.moves.configparser")
-    def test_get_config_value(self, mock_config_parser):
+    @mock.patch("os.path.exists")
+    def test_get_config_value(self, mock_path_exists, mock_config_parser):
 
+        mock_path_exists.return_value = True
         mock_config_parser.ConfigParser().get.return_value = "pa$$word"
 
         value = utils.get_config_value('section', 'password_name')
 
         self.assertEqual(value, "pa$$word")
 
+    @mock.patch("six.moves.configparser")
+    @mock.patch("os.path.exists")
+    def test_get_config_value_no_file(self, mock_path_exists,
+                                      mock_config_parser):
+
+        mock_path_exists.return_value = False
+        self.assertRaises(exceptions.PasswordFileNotFound,
+                          utils.get_config_value, 'section',
+                          'password_name')
+
     def test_wait_for_provision_state(self):
 
         baremetal_client = mock.Mock()
@@ -449,26 +418,27 @@ class TestCheckNodesCount(TestCase):
 
     def test_check_nodes_count_deploy_enough_nodes(self):
         user_params = {'ControllerCount': 2}
-        self.assertEqual(True,
+        self.assertEqual((True, 3, 3),
                          utils.check_nodes_count(self.baremetal, None,
                                                  user_params, self.defaults))
 
     def test_check_nodes_count_deploy_too_much(self):
         user_params = {'ControllerCount': 3}
-        self.assertRaises(exceptions.DeploymentError, utils.check_nodes_count,
-                          self.baremetal, None, user_params, self.defaults)
+        self.assertEqual((False, 4, 3),
+                         utils.check_nodes_count(self.baremetal, None,
+                                                 user_params, self.defaults))
 
     def test_check_nodes_count_scale_enough_nodes(self):
         user_params = {'ControllerCount': 2}
-        self.assertEqual(True,
-                         utils.check_nodes_count(self.baremetal, None,
+        self.assertEqual((True, 3, 3),
+                         utils.check_nodes_count(self.baremetal, self.stack,
                                                  user_params, self.defaults))
 
     def test_check_nodes_count_scale_too_much(self):
         user_params = {'ControllerCount': 3}
-        self.assertRaises(exceptions.DeploymentError, utils.check_nodes_count,
-                          self.baremetal, self.stack, user_params,
-                          self.defaults)
+        self.assertEqual((False, 4, 3),
+                         utils.check_nodes_count(self.baremetal, self.stack,
+                                                 user_params, self.defaults))
 
     def test_check_default_param_not_in_stack(self):
         missing_param = 'CephStorageCount'
