@@ -45,6 +45,8 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         # mock validations for all deploy tests
         # for validator tests, see test_overcloud_deploy_validators.py
         validator_mock = mock.Mock(return_value=(0, 0))
+        self.real_predeploy_verify_capabilities = \
+            self.cmd._predeploy_verify_capabilities
         self.cmd._predeploy_verify_capabilities = validator_mock
 
         self._get_passwords = generate_overcloud_passwords_mock
@@ -83,11 +85,9 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
     @mock.patch('tripleoclient.utils.get_config_value', autospec=True)
     @mock.patch('tripleoclient.utils.check_hypervisor_stats',
                 autospec=True)
-    @mock.patch('tripleoclient.utils.create_cephx_key',
-                autospec=True)
     @mock.patch('uuid.uuid1', autospec=True)
     @mock.patch('time.time', autospec=True)
-    def test_tht_scale(self, mock_time, mock_uuid1, mock_create_cephx_key,
+    def test_tht_scale(self, mock_time, mock_uuid1,
                        mock_check_hypervisor_stats, mock_get_key,
                        mock_create_env, generate_certs_mock,
                        mock_get_templte_contents, mock_process_multiple_env,
@@ -108,14 +108,13 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             ('ceph_storage_scale', 3)
         ]
 
-        mock_create_cephx_key.return_value = "cephx_key"
         mock_uuid1.return_value = "uuid"
         mock_time.return_value = 123456789
 
         mock_generate_overcloud_passwords.return_value = self._get_passwords()
 
         clients = self.app.client_manager
-        orchestration_client = clients.tripleoclient.orchestration
+        orchestration_client = clients.orchestration
         orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
         mock_event = mock.Mock()
         mock_event.id = '1234'
@@ -145,8 +144,14 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'AdminPassword': 'password',
             'AdminToken': 'password',
             'AodhPassword': 'password',
+            'BarbicanPassword': 'password',
             'CeilometerMeteringSecret': 'password',
             'CeilometerPassword': 'password',
+            'CephAdminKey': 'password',
+            'CephClientKey': 'password',
+            'CephClusterFSID': 'uuid',
+            'CephMonKey': 'password',
+            'CephRgwKey': 'password',
             'CephStorageCount': 3,
             'CinderPassword': 'password',
             'ExtraConfig': '{}',
@@ -157,12 +162,13 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'HeatStackDomainAdminPassword': 'password',
             'HypervisorNeutronPhysicalBridge': 'br-ex',
             'HypervisorNeutronPublicInterface': 'nic1',
+            'IronicPassword': 'password',
+            'ManilaPassword': 'password',
+            'MistralPassword': 'password',
             'MysqlClustercheckPassword': 'password',
-            'NeutronAllowL3AgentFailover': False,
             'NeutronDhcpAgentsPerNetwork': 1,
             'NeutronDnsmasqOptions': 'dhcp-option-force=26,1400',
             'NeutronFlatNetworks': 'datacentre',
-            'NeutronL3HA': False,
             'NeutronMetadataProxySharedSecret': 'password',
             'NeutronPassword': 'password',
             'NeutronPublicInterface': 'nic1',
@@ -174,8 +180,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'SnmpdReadonlyUserPassword': 'PASSWORD',
             'SwiftHashSuffix': 'password',
             'SwiftPassword': 'password',
+            'TrovePassword': 'password',
+            'ZaqarPassword': 'password',
             'DeployIdentifier': 123456789,
-            'UpdateIdentifier': None,
+            'UpdateIdentifier': '',
             'StackAction': 'UPDATE',
         }
 
@@ -189,8 +197,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         mock_create_parameters_env.side_effect = _custom_create_params_env
 
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
 
         args, kwargs = orchestration_client.stacks.update.call_args
 
@@ -241,11 +248,9 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
     @mock.patch('tripleoclient.utils.get_config_value', autospec=True)
     @mock.patch('tripleoclient.utils.check_hypervisor_stats',
                 autospec=True)
-    @mock.patch('tripleoclient.utils.create_cephx_key',
-                autospec=True)
     @mock.patch('uuid.uuid1', autospec=True)
     @mock.patch('time.time', autospec=True)
-    def test_tht_deploy(self, mock_time, mock_uuid1, mock_create_cephx_key,
+    def test_tht_deploy(self, mock_time, mock_uuid1,
                         mock_check_hypervisor_stats, mock_get_key,
                         mock_create_env, generate_certs_mock,
                         mock_get_templte_contents, mock_process_multiple_env,
@@ -265,14 +270,13 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             ('ceph_storage_scale', 3)
         ]
 
-        mock_create_cephx_key.return_value = "cephx_key"
         mock_uuid1.return_value = "uuid"
         mock_time.return_value = 123456789
 
         mock_generate_overcloud_passwords.return_value = self._get_passwords()
 
         clients = self.app.client_manager
-        orchestration_client = clients.tripleoclient.orchestration
+        orchestration_client = clients.orchestration
         mock_stack = fakes.create_tht_stack()
         orchestration_client.stacks.get.return_value = None
 
@@ -305,12 +309,14 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'AdminPassword': 'password',
             'AdminToken': 'password',
             'AodhPassword': 'password',
+            'BarbicanPassword': 'password',
             'CeilometerMeteringSecret': 'password',
             'CeilometerPassword': 'password',
-            'CephAdminKey': 'cephx_key',
-            'CephClientKey': 'cephx_key',
+            'CephAdminKey': 'password',
+            'CephClientKey': 'password',
             'CephClusterFSID': 'uuid',
-            'CephMonKey': 'cephx_key',
+            'CephMonKey': 'password',
+            'CephRgwKey': 'password',
             'CephStorageCount': 3,
             'CinderPassword': 'password',
             'ExtraConfig': '{}',
@@ -321,13 +327,13 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'HeatStackDomainAdminPassword': 'password',
             'HypervisorNeutronPhysicalBridge': 'br-ex',
             'HypervisorNeutronPublicInterface': 'nic1',
+            'IronicPassword': 'password',
+            'ManilaPassword': 'password',
+            'MistralPassword': 'password',
             'MysqlClustercheckPassword': 'password',
-            'NeutronAllowL3AgentFailover': False,
             'NeutronDhcpAgentsPerNetwork': 1,
             'NeutronDnsmasqOptions': 'dhcp-option-force=26,1400',
-            'NeutronEnableTunnelling': 'True',
             'NeutronFlatNetworks': 'datacentre',
-            'NeutronL3HA': False,
             'NeutronNetworkType': 'gre',
             'NeutronMetadataProxySharedSecret': 'password',
             'NeutronPassword': 'password',
@@ -341,8 +347,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             'SnmpdReadonlyUserPassword': 'PASSWORD',
             'SwiftHashSuffix': 'password',
             'SwiftPassword': 'password',
+            'TrovePassword': 'password',
+            'ZaqarPassword': 'password',
             'DeployIdentifier': 123456789,
-            'UpdateIdentifier': None,
+            'UpdateIdentifier': '',
             'StackAction': 'CREATE',
         }
 
@@ -356,8 +364,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
         mock_create_parameters_env.side_effect = _custom_create_params_env
 
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
 
         args, kwargs = orchestration_client.stacks.create.call_args
 
@@ -434,7 +441,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         mock_generate_overcloud_passwords.return_value = self._get_passwords()
 
         clients = self.app.client_manager
-        orchestration_client = clients.tripleoclient.orchestration
+        orchestration_client = clients.orchestration
         orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
         mock_events.return_value = []
 
@@ -462,8 +469,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             mkstemp.return_value = (os.open(self.parameter_defaults_env_file,
                                             os.O_RDWR),
                                     self.parameter_defaults_env_file)
-            result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+            self.cmd.take_action(parsed_args)
 
         args, kwargs = orchestration_client.stacks.update.call_args
 
@@ -485,10 +491,10 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
             [self.parameter_defaults_env_file])
 
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_deploy_tripleo_heat_templates', autospec=True)
+                'set_overcloud_passwords', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
-    def test_missing_sat_url(self, mock_pre_deploy, mock_deploy_tht):
+                '_deploy_tripleo_heat_templates', autospec=True)
+    def test_missing_sat_url(self, mock_deploy_tht, mock_set_ov_passwords):
 
         arglist = ['--templates', '--rhel-reg',
                    '--reg-method', 'satellite', '--reg-org', '123456789',
@@ -502,8 +508,9 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertFalse(result)
+        self.assertRaises(exceptions.DeploymentError,
+                          self.cmd.take_action,
+                          parsed_args)
         self.assertFalse(mock_deploy_tht.called)
 
     @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
@@ -515,11 +522,9 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                 '_deploy_postconfig', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_update_parameters', autospec=True)
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_heat_deploy', autospec=True)
-    def test_environment_dirs(self, mock_deploy_heat, mock_pre_heat,
+    def test_environment_dirs(self, mock_deploy_heat,
                               mock_update_parameters, mock_post_config,
                               mock_utils_check_nodes, mock_utils_endpoint,
                               mock_utils_createrc, mock_utils_tempest):
@@ -549,8 +554,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         mock_deploy_heat.side_effect = _fake_heat_deploy
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
 
     @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
                 autospec=True)
@@ -561,11 +565,9 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                 '_deploy_postconfig', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_update_parameters', autospec=True)
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_heat_deploy', autospec=True)
-    def test_environment_dirs_env(self, mock_deploy_heat, mock_pre_heat,
+    def test_environment_dirs_env(self, mock_deploy_heat,
                                   mock_update_parameters, mock_post_config,
                                   mock_utils_check_nodes, mock_utils_endpoint,
                                   mock_utils_createrc, mock_utils_tempest):
@@ -593,22 +595,22 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         mock_deploy_heat.side_effect = _fake_heat_deploy
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
         os.unlink(test_env)
         os.rmdir(tmp_dir)
 
     @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
                 autospec=True)
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                'set_overcloud_passwords', autospec=True)
     @mock.patch('tripleoclient.utils.create_overcloudrc', autospec=True)
     @mock.patch('tripleoclient.utils.get_overcloud_endpoint', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_deploy_tripleo_heat_templates', autospec=True)
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
-    def test_rhel_reg_params_provided(self, mock_pre_deploy, mock_deploy_tht,
+    def test_rhel_reg_params_provided(self, mock_deploy_tht,
                                       mock_oc_endpoint,
                                       mock_create_ocrc,
+                                      mock_set_oc_passwords,
                                       mock_create_tempest_deployer_input):
 
         arglist = ['--templates', '--rhel-reg',
@@ -625,8 +627,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
         self.assertTrue(mock_deploy_tht.called)
         self.assertTrue(mock_oc_endpoint.called)
         self.assertTrue(mock_create_ocrc.called)
@@ -693,7 +694,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         wait_for_stack_ready_mock.return_value = True
 
         clients = self.app.client_manager
-        orchestration_client = clients.tripleoclient.orchestration
+        orchestration_client = clients.orchestration
         orchestration_client.stacks.get.return_value = fakes.create_tht_stack()
         mock_events.return_value = []
 
@@ -712,8 +713,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         baremetal = clients.baremetal
         baremetal.node.list.return_value = range(10)
 
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
 
         args, kwargs = mock_process_multiple_env.call_args
         self.assertIn(
@@ -754,6 +754,20 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                           self.cmd._validate_args,
                           parsed_args)
 
+    def test_validate_args_missing_environment_files(self):
+        arglist = ['--templates',
+                   '-e', 'nonexistent.yaml']
+        verifylist = [
+            ('templates', '/usr/share/openstack-tripleo-heat-templates/'),
+            ('environment_files', ['nonexistent.yaml']),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(oscexc.CommandError,
+                          self.cmd._validate_args,
+                          parsed_args)
+
     def test_validate_args_no_tunnel_type(self):
         arglist = ['--templates',
                    '--neutron-network-type', 'nettype']
@@ -826,23 +840,6 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                           self.cmd._validate_args,
                           parsed_args)
 
-    @mock.patch('tripleoclient.utils.check_hypervisor_stats',
-                autospec=True)
-    def test_pre_heat_deploy_failed(self, mock_check_hypervisor_stats):
-        clients = self.app.client_manager
-        orchestration_client = clients.tripleoclient.orchestration
-        orchestration_client.stacks.get.return_value = None
-        mock_check_hypervisor_stats.return_value = None
-        arglist = ['--templates']
-        verifylist = [
-            ('templates', '/usr/share/openstack-tripleo-heat-templates/')
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertFalse(result)
-        self.assertRaises(exceptions.DeploymentError,
-                          self.cmd._pre_heat_deploy)
-
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_heat_deploy', autospec=True)
     def test_try_overcloud_deploy_with_first_template_existing(
@@ -902,15 +899,16 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
 
     @mock.patch('tripleoclient.utils.create_tempest_deployer_input',
                 autospec=True)
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                'set_overcloud_passwords', autospec=True)
     @mock.patch('tripleoclient.utils.create_overcloudrc', autospec=True)
     @mock.patch('tripleoclient.utils.get_overcloud_endpoint', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 '_deploy_tripleo_heat_templates', autospec=True)
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
-    def test_dry_run(self, mock_pre_deploy, mock_deploy_tht,
+    def test_dry_run(self, mock_deploy_tht,
                      mock_oc_endpoint,
                      mock_create_ocrc,
+                     mock_set_ov_passwords,
                      mock_create_tempest_deployer_input):
 
         arglist = ['--templates', '--dry-run']
@@ -920,8 +918,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
+        self.cmd.take_action(parsed_args)
         self.assertFalse(mock_deploy_tht.called)
         self.assertFalse(mock_oc_endpoint.called)
         self.assertFalse(mock_create_ocrc.called)
@@ -937,10 +934,7 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                 '_heat_deploy', autospec=True)
     @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
                 'set_overcloud_passwords', autospec=True)
-    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
-                '_pre_heat_deploy', autospec=True)
     def test_answers_file(self,
-                          mock_pre_deploy,
                           mock_set_overcloud_passwords,
                           mock_heat_deploy,
                           mock_oc_endpoint,
@@ -954,26 +948,27 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
         net.configure_mock(__getitem__=lambda x, y: 'testnet')
 
         with tempfile.NamedTemporaryFile(mode="w+t") as answerfile:
-            yaml.dump(
-                {'templates': '/dev/null',
-                 'environments': ['/tmp/foo.yaml']
-                 },
-                answerfile
-            )
-            answerfile.flush()
+            with open('/tmp/environment.yaml', "w+t") as environmentfile:
+                yaml.dump(
+                    {'templates': '/dev/null',
+                     'environments': ['/tmp/foo.yaml']
+                     },
+                    answerfile
+                )
+                answerfile.flush()
 
-            arglist = ['--answers-file', answerfile.name,
-                       '--environment-file', '/tmp/environment.yaml',
-                       '--block-storage-scale', '3']
-            verifylist = [
-                ('answers_file', answerfile.name),
-                ('environment_files', ['/tmp/environment.yaml']),
-                ('block_storage_scale', 3)
-            ]
-            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+                arglist = ['--answers-file', answerfile.name,
+                           '--environment-file', environmentfile.name,
+                           '--block-storage-scale', '3']
+                verifylist = [
+                    ('answers_file', answerfile.name),
+                    ('environment_files', [environmentfile.name]),
+                    ('block_storage_scale', 3)
+                ]
+                parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+                self.cmd.take_action(parsed_args)
 
-            result = self.cmd.take_action(parsed_args)
-        self.assertTrue(result)
         self.assertTrue(mock_heat_deploy.called)
         self.assertTrue(mock_oc_endpoint.called)
         self.assertTrue(mock_create_ocrc.called)
@@ -1047,3 +1042,31 @@ class TestDeployOvercloud(fakes.TestDeployOvercloud):
                                                                     args,
                                                                     stack)
         self.assertTrue(mock_init.called)
+
+    @mock.patch('tripleoclient.utils.check_nodes_count')
+    @mock.patch('tripleoclient.utils.check_hypervisor_stats')
+    @mock.patch('tripleoclient.utils.assign_and_verify_profiles')
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                '_check_ironic_boot_configuration')
+    @mock.patch('tripleoclient.v1.overcloud_deploy.DeployOvercloud.'
+                '_collect_flavors')
+    def test_predeploy_verify_capabilities_hypervisor_stats(
+            self, mock_collect_flavors,
+            mock_check_ironic_boot_configuration,
+            mock_assign_and_verify_profiles,
+            mock_check_hypervisor_stats,
+            mock_check_nodes_count):
+        self.cmd._predeploy_verify_capabilities = \
+            self.real_predeploy_verify_capabilities
+
+        stack = None
+        parameters = {}
+        parsed_args = mock.Mock()
+        mock_assign_and_verify_profiles.return_value = (0, 0)
+        mock_check_nodes_count.return_value = (True, 0, 0)
+
+        # A None return value here indicates an error
+        mock_check_hypervisor_stats.return_value = None
+        self.cmd._predeploy_verify_capabilities(
+            stack, parameters, parsed_args)
+        self.assertEqual(1, self.cmd.predeploy_errors)
